@@ -61,7 +61,7 @@ except ImportError:
 try:
     import pyspread.commands as commands
     from pyspread.dialogs import DiscardDataDialog
-    from pyspread.grid_renderer import painter_save, CellRenderer
+    from pyspread.grid_renderer import painter_save, CellRenderer, QColorCache
     from pyspread.model.model import (CodeArray, CellAttribute,
                                       DefaultCellAttributeDict)
     from pyspread.lib.attrdict import AttrDict
@@ -77,7 +77,7 @@ try:
 except ImportError:
     import commands
     from dialogs import DiscardDataDialog
-    from grid_renderer import painter_save, CellRenderer
+    from grid_renderer import painter_save, CellRenderer, QColorCache
     from model.model import CodeArray, CellAttribute, DefaultCellAttributeDict
     from lib.attrdict import AttrDict
     from lib.selection import Selection
@@ -105,6 +105,8 @@ class Grid(QTableView):
 
         self.main_window = main_window
 
+        self.qcolor_cache = QColorCache(self)
+
         shape = main_window.settings.shape
 
         if model is None:
@@ -298,6 +300,8 @@ class Grid(QTableView):
 
         block_top_left = []
         block_bottom_right = []
+        rows = []
+        columns = []
         cells = []
 
         # Selection are made of selection ranges that we call span
@@ -306,15 +310,22 @@ class Grid(QTableView):
             top, bottom = span.top(), span.bottom()
             left, right = span.left(), span.right()
 
-            # If the span is a single cell then append it
             if top == bottom and left == right:
+                # The span is a single cell
                 cells.append((top, right))
+            elif left == 0 and right == self.model.shape[1] - 1:
+                # The span consists of selected rows
+                rows += list(range(top, bottom + 1))
+            elif top == 0 and bottom == self.model.shape[0] - 1:
+                # The span consists of selected columns
+                columns += list(range(left, right + 1))
             else:
                 # Otherwise append a block
                 block_top_left.append((top, left))
                 block_bottom_right.append((bottom, right))
 
-        return Selection(block_top_left, block_bottom_right, [], [], cells)
+        return Selection(block_top_left, block_bottom_right,
+                         rows, columns, cells)
 
     @property
     def selected_idx(self) -> List[QModelIndex]:
@@ -380,6 +391,8 @@ class Grid(QTableView):
     def set_selection_mode(self, value=True):
         """Setter for selection mode
 
+        This method is required for accessing selection mode from QActions.
+
         :param value: If True, selection mode is set, if False unset
 
         """
@@ -1143,9 +1156,11 @@ class Grid(QTableView):
 
         border_choice = self.main_window.settings.border_choice
         bottom_selection = \
-            self.selection.get_bottom_borders_selection(border_choice)
+            self.selection.get_bottom_borders_selection(border_choice,
+                                                        self.model.shape)
         right_selection = \
-            self.selection.get_right_borders_selection(border_choice)
+            self.selection.get_right_borders_selection(border_choice,
+                                                       self.model.shape)
 
         line_color = self.main_window.widgets.line_color_button.color
         line_color_rgb = line_color.getRgb()
@@ -1191,9 +1206,11 @@ class Grid(QTableView):
 
         border_choice = self.main_window.settings.border_choice
         bottom_selection = \
-            self.selection.get_bottom_borders_selection(border_choice)
+            self.selection.get_bottom_borders_selection(border_choice,
+                                                        self.model.shape)
         right_selection = \
-            self.selection.get_right_borders_selection(border_choice)
+            self.selection.get_right_borders_selection(border_choice,
+                                                       self.model.shape)
 
         attr_dict_bottom = AttrDict([("borderwidth_bottom", width)])
         attr_bottom = CellAttribute(bottom_selection, self.table,
@@ -1331,7 +1348,10 @@ class Grid(QTableView):
 
         # This is not done in the model because setSpan does not work there
 
-        bbox = self.selection.get_grid_bbox(self.model.shape)
+        shape = list(self.model.shape)
+        shape[0] -= 1
+        shape[1] -= 1
+        bbox = self.selection.get_grid_bbox(shape)
         (top, left), (bottom, right) = bbox
 
         # Check if current cell is already merged
@@ -1374,7 +1394,7 @@ class Grid(QTableView):
             self.main_window.undo_stack.push(command)
 
     def is_row_data_discarded(self, count: int) -> bool:
-        """Shows user dialog if data is to be discarded
+        """True if row data is to be discarded on row insertion
 
         :param count: Rows to be inserted
 
@@ -1388,7 +1408,7 @@ class Grid(QTableView):
                    for key in sel_cell_gen)
 
     def is_column_data_discarded(self, count: int) -> bool:
-        """Shows user dialog if data is to be discarded
+        """True if column data is to be discarded on column insertion
 
         :param count: Columns to be inserted
 
@@ -1402,7 +1422,7 @@ class Grid(QTableView):
                    for key in sel_cell_gen)
 
     def is_table_data_discarded(self, count: int) -> bool:
-        """Shows user dialog if data is to be discarded
+        """True if table data is to be discarded  on table insertion
 
         :param count: Tables to be inserted
 
@@ -1544,7 +1564,7 @@ class GridHeaderView(QHeaderView):
         return QSize(int(unzoomed_size.width() * self.grid.zoom),
                      int(unzoomed_size.height() * self.grid.zoom))
 
-    def sectionSizeHint(self, logicalIndex: int) -> QSize:
+    def sectionSizeHint(self, logicalIndex: int) -> int:
         """Overrides sectionSizeHint, which supports zoom
 
         :param logicalIndex: Index of the section for the size hint
@@ -1552,8 +1572,7 @@ class GridHeaderView(QHeaderView):
         """
 
         unzoomed_size = super().sectionSizeHint(logicalIndex)
-        return QSize(int(unzoomed_size.width() * self.grid.zoom),
-                     int(unzoomed_size.height() * self.grid.zoom))
+        return int(unzoomed_size * self.grid.zoom)
 
     def paintSection(self, painter: QPainter, rect: QRect, logicalIndex: int):
         """Overrides paintSection, which supports zoom
@@ -1565,8 +1584,8 @@ class GridHeaderView(QHeaderView):
         """
 
         unzoomed_rect = QRect(0, 0,
-                              rect.width()/self.grid.zoom,
-                              rect.height()/self.grid.zoom)
+                              rect.width()//self.grid.zoom,
+                              rect.height()//self.grid.zoom)
         with painter_save(painter):
             painter.translate(rect.x(), rect.y())
             painter.scale(self.grid.zoom, self.grid.zoom)
@@ -1605,7 +1624,7 @@ class GridHeaderView(QHeaderView):
                     section_sizes = self.grid.row_heights
 
                 for section, size in section_sizes:
-                    self.resizeSection(section, size * self.grid.zoom)
+                    self.resizeSection(section, int(size * self.grid.zoom))
 
 
 class GridTableModel(QAbstractTableModel):
@@ -1955,7 +1974,9 @@ class GridTableModel(QAbstractTableModel):
                     self.code_array[key] = value
             else:
                 self.code_array[key] = "{}".format(value)
-            self.dataChanged.emit(index, index)
+
+            if not self.main_window.prevent_updates:
+                self.dataChanged.emit(index, index)
 
             return True
 
@@ -2108,7 +2129,7 @@ class GridCellDelegate(QStyledItemDelegate):
 
     def _render_text(self, painter: QPainter, rect: QRectF,
                      option: QStyleOptionViewItem, index: QModelIndex):
-        """HTML markup renderer
+        """Text renderer
 
         :param painter: Painter with which markup is rendered
         :param rect: Cell rect of the cell to be painted
