@@ -36,6 +36,7 @@ pyspread
 
 import os
 import sys
+import traceback
 
 from PyQt5.QtCore import Qt, pyqtSignal, QEvent, QTimer, QRectF
 from PyQt5.QtWidgets import (QWidget, QMainWindow, QApplication,
@@ -51,7 +52,7 @@ from PyQt5.QtPrintSupport import QPrinter, QPrintDialog
 try:
     from pyspread.__init__ import VERSION, APP_NAME
     from pyspread.cli import PyspreadArgumentParser
-    from pyspread.settings import Settings
+    from pyspread.settings import Settings, WEB_URL
     from pyspread.icons import Icon, IconPath
     from pyspread.grid import Grid
     from pyspread.grid_renderer import painter_save
@@ -72,7 +73,7 @@ try:
 except ImportError:
     from __init__ import VERSION, APP_NAME
     from cli import PyspreadArgumentParser
-    from settings import Settings
+    from settings import Settings, WEB_URL
     from icons import Icon, IconPath
     from grid import Grid
     from grid_renderer import painter_save
@@ -102,18 +103,19 @@ class MainWindow(QMainWindow):
 
     gui_update = pyqtSignal(dict)
 
-    def __init__(self, filepath: str = None, reset_settings: bool = False):
+    def __init__(self, filepath: str = None, default_settings: bool = False):
         """
         :param filepath: File path for inital file to be opened
-        :param reset_settings: Ignore stored `QSettings` and use defaults
+        :param default_settings: Ignore stored `QSettings` and use defaults
 
         """
 
         super().__init__()
 
-        self._loading = True
+        self._loading = True  # For initial loading of pyspread
+        self.prevent_updates = False  # Prevents setData updates in grid
 
-        self.settings = Settings(self, reset_settings=reset_settings)
+        self.settings = Settings(self, reset_settings=default_settings)
         self.workflows = Workflows(self)
         self.undo_stack = QUndoStack(self)
         self.refresh_timer = QTimer()
@@ -660,25 +662,42 @@ class MainWindow(QMainWindow):
     def on_about(self):
         """Show about message box"""
 
-        about_msg_template = "<p>".join((
-            "<b>%s</b>" % APP_NAME,
-            "A non-traditional Python spreadsheet application",
-            "Version {version}",
-            "Created by:<br>{devs}",
-            "Documented by:<br>{doc_devs}",
-            "Copyright:<br>Martin Manns",
-            "License:<br>{license}",
-            '<a href="https://pyspread.gitlab.io">pyspread.gitlab.io</a>',
-            ))
+        def devs_string(devs: list) -> str:
+            """Get string from devs list"""
 
-        devs = "Martin Manns, Jason Sexauer<br>Vova Kolobok, mgunyho, " \
-               "Pete Morgan"
+            devs_str = "".join("<li>{}</li>".format(dev) for dev in devs)
+            return "<ul>{}</ul>".format(devs_str)
 
-        doc_devs = "Martin Manns, Bosko Markovic, Pete Morgan"
+        about_msg_template = \
+            """<b>{name}</b><><p>
+            A non-traditional Python spreadsheet application<p>
+            Version:&emsp;{version}<p>
+            Created by:&emsp;{devs}<p>
+            Documented by:&emsp;{doc_devs}<p>
+            Copyright:&emsp;{copyright_owner}<p>
+            License:&emsp;{license}<p>
+            Web site:&emsp;<a href="{web_url}">{web_url}</a>
+            """
 
-        about_msg = about_msg_template.format(version=VERSION, license=LICENSE,
-                                              devs=devs, doc_devs=doc_devs)
-        QMessageBox.about(self, "About %s" % APP_NAME, about_msg)
+        devs = ("Martin Manns", "Jason Sexauer", "Vova Kolobok", "mgunyho",
+                "Pete Morgan")
+        devs_str = devs_string(devs)
+
+        doc_devs = ("Martin Manns", "Bosko Markovic", "Pete Morgan")
+        doc_devs_str = devs_string(doc_devs)
+
+        copyright_owner = "Martin Manns"
+
+        about_msg = about_msg_template.format(
+            name=APP_NAME,
+            version=VERSION,
+            license=LICENSE,
+            devs=devs_str,
+            doc_devs=doc_devs_str,
+            copyright_owner=copyright_owner,
+            web_url=WEB_URL)
+
+        QMessageBox.about(self, "About {}".format(APP_NAME), about_msg)
 
     def on_focus_changed(self, old: QWidget, now: QWidget):
         """Handles grid clicks from entry line"""
@@ -765,19 +784,32 @@ class MainWindow(QMainWindow):
         merge_cells_action.setChecked(attributes.merge_area is not None)
 
 
+def excepthook(exception_type, exception_value, exception_traceback):
+    """Exception hook that prevents pyspread from crashing on exceptions"""
+
+    traceback_msg = "".join(traceback.format_exception(exception_type,
+                                                       exception_value,
+                                                       exception_traceback))
+    print("Error: {}\n".format(traceback_msg))
+
+
 def main():
     """Pyspread main"""
 
+    sys.excepthook = excepthook
+
     parser = PyspreadArgumentParser()
     args, unknown = parser.parse_known_args()
 
     app = QApplication(sys.argv)
-    main_window = MainWindow(args.file, reset_settings=args.reset_settings)
+    main_window = MainWindow(args.file, default_settings=args.default_settings)
 
     main_window.show()
 
     app.exec_()
 
+    sys.exit()
+
 
 if __name__ == '__main__':
     main()
