@@ -42,6 +42,11 @@
 from pathlib import Path
 from typing import Tuple
 
+try:
+    from markdown2 import markdown
+except ImportError:
+    markdown = None
+
 from PyQt5.QtCore import pyqtSignal, QSize, Qt, QModelIndex, QPoint
 from PyQt5.QtWidgets \
     import (QToolButton, QColorDialog, QFontComboBox, QComboBox, QSizePolicy,
@@ -52,11 +57,11 @@ from PyQt5.QtGui import QPalette, QColor, QFont, QIntValidator, QCursor, QIcon
 try:
     from pyspread.actions import Action
     from pyspread.icons import Icon
-    from pyspread.lib.markdown2 import markdown
+    from pyspread.settings import WEB_URL
 except ImportError:
     from actions import Action
     from icons import Icon
-    from lib.markdown2 import markdown
+    from settings import WEB_URL
 
 
 class MultiStateBitmapButton(QToolButton):
@@ -714,5 +719,10 @@ class HelpBrowser(QTextBrowser):
         except IOError as err:
             return "Error opening file {}: {}".format(path, err)
 
+        if markdown is None:
+            error_msg = "<b>Warning: markdown2 is not installed.<br>" + \
+                        "Rendering as pain text.</b><p>"
+            return error_msg + help_text.replace("\n", "<br>")
+
         return markdown(help_text, extras=['metadata', 'code-friendly',
                                            'fenced-code-blocks'])
